/* --COPYRIGHT--,BSD
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
// Header files for driverlib
#include "driverlib.h"

// Header files for IEC
#include "IEC60730_cpu_test.h"
#include "IEC60730_pc_test.h"
#include "IEC60730_user_config.h"
#include "IEC60730_system_config.h"
#include "IEC60730_clock_fail_test.h"
#include "IEC60730_march_test.h"
#include "IEC60730_crc_test.h"
#include "IEC60730_adc_test.h"
#include "IEC60730_gpio_test.h"

//*****************************************************************************
//
//Target frequency for MCLK in kHz
//
//*****************************************************************************
#if defined(MAIN_CLOCK_FREQUENCY_1MHz)
#define CS_MCLK_DESIRED_FREQUENCY_IN_KHZ   1000
#define CS_MCLK_FLLREF_RATIO   31
#elif defined(MAIN_CLOCK_FREQUENCY_8MHz)
#define CS_MCLK_DESIRED_FREQUENCY_IN_KHZ   8000
#define CS_MCLK_FLLREF_RATIO   244
#elif defined(MAIN_CLOCK_FREQUENCY_12MHz)
#define CS_MCLK_DESIRED_FREQUENCY_IN_KHZ   12000
#define CS_MCLK_FLLREF_RATIO   366
#endif

#define GPIO_PIN_ALL 	GPIO_PIN0 | GPIO_PIN1 | GPIO_PIN2 | GPIO_PIN3 | \
						GPIO_PIN4 | GPIO_PIN5 | GPIO_PIN6 | GPIO_PIN7

#define TEST_CAPTIVATE_INTERRUPT                  		0x00000001
#define TEST_PORT2_INTERRUPT                      		0x00000002
#define TEST_PORT1_INTERRUPT                      		0x00000004
#define TEST_ADC_INTERRUPT                        		0x00000008
#define TEST_USCI_B0_INTERRUPT                    		0x00000010
#define TEST_USCI_A1_INTERRUPT                    		0x00000020
#define TEST_USCI_A0_INTERRUPT                    		0x00000040
#define TEST_WDT_INTERRUPT                        		0x00000080
#define TEST_RTC_INTERRUPT                         		0x00000100
#define TEST_TIMER3_A1_INTERRUPT                   		0x00000200
#define TEST_TIMER3_A0_INTERRUPT                   		0x00000400
#define TEST_TIMER2_A1_INTERRUPT                   		0x00000800
#define TEST_TIMER2_A0_INTERRUPT                   		0x00001000
#define TEST_TIMER1_A1_INTERRUPT                  		0x00002000
#define TEST_TIMER1_A0_INTERRUPT                  		0x00004000
#define TEST_TIMER0_A1_INTERRUPT                  		0x00008000
#define TEST_TIMER0_A0_INTERRUPT                  		0x00010000
#define TEST_UNMI_INTERRUPT                       		0x00020000
#define TEST_SYSNMI_INTERRUPT                     		0x00040000


// Defines for to determine failure
#define CPU_TEST_FAILURE_BLINK			1
#define PC_TEST_FAILURE_BLINK			2
#define OSCILLATOR_TEST_FAILURE_BLINK	3
#define MARCH_TEST_FAILURE_BLINK		4
#define CRC_TEST_FAILURE_BLINK			5
#define INTERRUPT_TEST_FAILURE_BLINK	6
#define ADC_TEST_FAILURE_BLINK			7
#define GPIO_INPUT_TEST_FAILURE_BLINK	8
#define GPIO_OUTPUT_TEST_FAILURE_BLINK	9


// Define to handle failure of an IEC60730 Class B test
void IEC60730_FAIL_SAFE_failSafe(uint8_t blinks);

uint16_t IEC60730_INTERRUPT_TEST_testInterrupt(uint32_t interruptTestRegister);


void configureWDT(void);
void initGPIO(void);
void initCLK(void);
void initADC(void);
void initREF(void);
void runPOST(void);
void runPST(void);

uint32_t gInterruptCounter=0;

void main(void){

	configureWDT();
	initGPIO();
	initCLK();
	initADC();


	runPOST();


	for(;;){

		runPST();

	}
}

void IEC60730_FAIL_SAFE_failSafe(uint8_t blinks){
	uint8_t i=0;
	GPIO_setOutputLowOnPin(GPIO_PORT_P1,
							GPIO_PIN6);
	for(i=blinks;i>0;i--){
		GPIO_setOutputHighOnPin(GPIO_PORT_P1, GPIO_PIN7);
		__delay_cycles(MAIN_CLOCK_FREQUENCY * 0.5);
		GPIO_setOutputLowOnPin(GPIO_PORT_P1, GPIO_PIN7);
		__delay_cycles(MAIN_CLOCK_FREQUENCY * 0.5);
		WDT_A_resetTimer(WDT_A_BASE);
	}

	for(;;);

}

void runPST(void){
	if(IEC60730_PC_TEST_testPcRegister()==TEST_FAILURE){

			IEC60730_FAIL_SAFE_failSafe(PC_TEST_FAILURE_BLINK);

	}

	if(IEC60730_OSCILLATOR_TEST_testOsc()==TEST_FAILURE){

	IEC60730_FAIL_SAFE_failSafe(OSCILLATOR_TEST_FAILURE_BLINK);

	}

	if(IEC60730_MARCH_TEST_testRam((uint16_t*) RAM_START_ADDRESS , (uint16_t*)RAM_END_ADDRESS)==TEST_FAILURE){

	IEC60730_FAIL_SAFE_failSafe(MARCH_TEST_FAILURE_BLINK);

	}
}


void runPOST(void){
	IEC60730_ADC_TEST_adcTest_Handle adcTestHandle;

	if(IEC60730_CPU_TEST_testCpuRegisters()==TEST_FAILURE){
			IEC60730_FAIL_SAFE_failSafe(CPU_TEST_FAILURE_BLINK);

	}

	if(IEC60730_PC_TEST_testPcRegister()==TEST_FAILURE){
			IEC60730_FAIL_SAFE_failSafe(PC_TEST_FAILURE_BLINK);

	}

	if(IEC60730_OSCILLATOR_TEST_testOsc()==TEST_FAILURE){
		IEC60730_FAIL_SAFE_failSafe(OSCILLATOR_TEST_FAILURE_BLINK);

	}

	if(IEC60730_MARCH_TEST_testRam((uint16_t*) RAM_START_ADDRESS ,
			(uint16_t*)RAM_END_ADDRESS)==TEST_FAILURE){
		IEC60730_FAIL_SAFE_failSafe(MARCH_TEST_FAILURE_BLINK);
	}

	if(IEC60730_INTERRUPT_TEST_testInterrupt(
								 TEST_PORT2_INTERRUPT     |
								 TEST_PORT1_INTERRUPT     |
								 TEST_ADC_INTERRUPT       |
								 TEST_USCI_B0_INTERRUPT   |
								 TEST_USCI_A1_INTERRUPT   |
								 TEST_USCI_A0_INTERRUPT   |
								 TEST_WDT_INTERRUPT       |
								 TEST_RTC_INTERRUPT       |
								 TEST_TIMER3_A1_INTERRUPT |
								 TEST_TIMER3_A0_INTERRUPT |
								 TEST_TIMER2_A1_INTERRUPT |
								 TEST_TIMER2_A0_INTERRUPT |
								 TEST_TIMER1_A1_INTERRUPT |
								 TEST_TIMER1_A0_INTERRUPT |
								 TEST_TIMER0_A1_INTERRUPT |
								 TEST_TIMER0_A0_INTERRUPT |
								 TEST_UNMI_INTERRUPT      |
								 TEST_SYSNMI_INTERRUPT)==TEST_FAILURE){

		IEC60730_FAIL_SAFE_failSafe(INTERRUPT_TEST_FAILURE_BLINK);

	}

	if(IEC60730_CRC_TEST_testNvMemory((uint16_t*) 0x4000, 0x3000,
			(uint16_t*) CRC_CHECKSUM_LOCATION)==TEST_FAILURE){

		IEC60730_FAIL_SAFE_failSafe(CRC_TEST_FAILURE_BLINK);

	}



	ADC_disableConversions(ADC_BASE,false);
    ADC_configureMemory(ADC_BASE,
    					15,
						ADC_VREFPOS_INT,
						ADC_VREFNEG_AVSS);

	adcTestHandle.muxChannel=15;
	adcTestHandle.pinCount=0x3FF;
	adcTestHandle.useInternalInput=INT_REF_1_5_V;


	if(IEC60730_ADC_TEST_testAdcInput(&adcTestHandle)==TEST_FAILURE){

			IEC60730_FAIL_SAFE_failSafe(ADC_TEST_FAILURE_BLINK);

	}

	if(IEC60730_GPIO_TEST_testGpioOutput(PORT_1,
    		PIN0|PIN1|PIN2|PIN3|PIN4|PIN5|PIN6|PIN7) == TEST_FAILURE){

			IEC60730_FAIL_SAFE_failSafe(GPIO_OUTPUT_TEST_FAILURE_BLINK);

	}

	GPIO_setAsInputPinWithPullDownResistor(GPIO_PORT_P1,
			GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3| \
			GPIO_PIN4);

	if(IEC60730_GPIO_TEST_testGpioInput(PORT_1,
			PIN0|PIN1|PIN2|PIN3|PIN4,
			PIN0_LOW|PIN1_LOW|PIN2_LOW|PIN3_LOW| \
			PIN4_LOW)==TEST_FAILURE){

	IEC60730_FAIL_SAFE_failSafe(GPIO_INPUT_TEST_FAILURE_BLINK);

	}
}

void configureWDT(void){

#if ENABLED_WDT
	// Start Watchdog timer on interval mode using ACLK/32K to generate a 1 sec Watchdog interval.
	WDT_A_initWatchdogTimer(WDT_A_BASE,WDT_A_CLOCKSOURCE_ACLK,WDT_A_CLOCKDIVIDER_32K);
	WDT_A_start(WDT_A_BASE);
#else
	WDT_A_hold(WDT_A_BASE);
#endif
}

void initGPIO(void){


	// Initialize all device pin to output direction and low
	GPIO_setOutputLowOnPin(GPIO_PORT_P1,
			GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5| \
			GPIO_PIN6|GPIO_PIN7);
	GPIO_setOutputLowOnPin(GPIO_PORT_P2,
			GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5| \
						GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P1,
			GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5| \
						GPIO_PIN6|GPIO_PIN7);
	GPIO_setAsOutputPin(GPIO_PORT_P2,
			GPIO_PIN0|GPIO_PIN1|GPIO_PIN2|GPIO_PIN3|GPIO_PIN4|GPIO_PIN5| \
						GPIO_PIN6|GPIO_PIN7);

    // Configure XIN and XOUT pins
	GPIO_setAsPeripheralModuleFunctionOutputPin(
			GPIO_PORT_P2,GPIO_PIN0,GPIO_PRIMARY_MODULE_FUNCTION);
	GPIO_setAsPeripheralModuleFunctionInputPin(
			GPIO_PORT_P2,GPIO_PIN1,GPIO_PRIMARY_MODULE_FUNCTION);

	 PMM_unlockLPM5();
}

void initCLK(){

    //Initializes the XT1 crystal oscillator with no timeout
    //In case of failure, code hangs here.
    //For time-out instead of code hang use UCS_LFXT1StartWithTimeout()
	CS_setExternalClockSource(32768);
    CS_turnOnXT1(CS_XT1_DRIVE_1);
	CS_initClockSignal(CS_ACLK,CS_XT1CLK_SELECT,CS_CLOCK_DIVIDER_1);

    //Set DCO FLL reference = REFO
    CS_initClockSignal(CS_FLLREF, CS_REFOCLK_SELECT, CS_CLOCK_DIVIDER_1);

    //Set Ratio and Desired MCLK Frequency  and initialize DCO
    CS_initFLLSettle(CS_MCLK_DESIRED_FREQUENCY_IN_KHZ,CS_MCLK_FLLREF_RATIO);

    //Clear all OSC fault flag
    CS_clearAllOscFlagsWithTimeout(1000);
}

void initADC(void){

    //Initialize the ADC Module
    /*
     * Base Address for the ADC Module
     * Use internal ADC bit as sample/hold signal to start conversion
     * USE MODOSC 5MHZ Digital Oscillator as clock source
     * Use default clock divider of 1
     */
    ADC_init(ADC_BASE,
             ADC_SAMPLEHOLDSOURCE_SC,
             ADC_CLOCKSOURCE_ADCOSC,
             ADC_CLOCKDIVIDER_1);

    ADC_enable(ADC_BASE);

    /*
     * Base Address for the ADC Module
     * Sample/hold for 16 clock cycles
     * Do not enable Multiple Sampling
     */
    ADC_setupSamplingTimer(ADC_BASE,
                           ADC_CYCLEHOLD_16_CYCLES,
                           ADC_MULTIPLESAMPLESDISABLE);


    //Configure internal reference
    //If ref voltage no ready, WAIT
    while(PMM_REFGEN_NOTREADY ==
          PMM_getVariableReferenceVoltageStatus())
    {
        ;
    }

    //Internal Reference ON
    PMM_enableInternalReference();

    // delay for reference settling ~80us
    __delay_cycles(1000000*0.000080);
}


uint16_t IEC60730_INTERRUPT_TEST_testInterrupt(uint32_t interruptTestRegister){

	uint16_t gieStatus = (__get_SR_register() & GIE);


	__bis_SR_register(GIE);


	if(interruptTestRegister&TEST_PORT2_INTERRUPT){
		// Enable GPIO interrupt on P2.7
		GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN_ALL );
		GPIO_enableInterrupt(GPIO_PORT_P2, GPIO_PIN7);
		// Trigger interrupt manually
		P2IFG |= (BIT7);
	}


	if(interruptTestRegister&TEST_PORT1_INTERRUPT){
		// Enable GPIO interrupt on P1.7
		GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN_ALL );
		GPIO_enableInterrupt(GPIO_PORT_P1, GPIO_PIN7);
		// Trigger interrupt manually
		P1IFG |= (BIT7);

	}


	if(interruptTestRegister&TEST_ADC_INTERRUPT){
		// Enable ADC interrupt on
		ADC_clearInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT_FLAG);
		ADC_enableInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT);
		// Trigger interrupt manually
		ADCIFG |= (ADCIFG0);
	}


	if(interruptTestRegister&TEST_USCI_B0_INTERRUPT){
		// Initialize eUSCI state machine
		EUSCI_B_SPI_enable(EUSCI_B0_BASE);
		// Trigger interrupt manually
		EUSCI_B_SPI_enableInterrupt(EUSCI_B0_BASE,
				EUSCI_B_SPI_TRANSMIT_INTERRUPT);
	}

	if(interruptTestRegister&TEST_USCI_A1_INTERRUPT){
		// Initialize eUSCI state machine
		EUSCI_A_SPI_enable(EUSCI_A1_BASE);
		// Trigger interrupt manually
		EUSCI_A_SPI_enableInterrupt(EUSCI_A1_BASE,
				EUSCI_A_SPI_TRANSMIT_INTERRUPT);

	}


	if(interruptTestRegister&TEST_USCI_A0_INTERRUPT){
		// Initialize eUSCI state machine
		EUSCI_A_SPI_enable(EUSCI_A0_BASE);
		// Trigger interrupt manually
		EUSCI_A_SPI_enableInterrupt(EUSCI_A0_BASE,
				EUSCI_A_SPI_TRANSMIT_INTERRUPT);
	}


	if(interruptTestRegister&TEST_WDT_INTERRUPT){
		// Set-up watch dog in interval to be able to jump to WDT ISR.
		WDT_A_initIntervalTimer(WDT_A_BASE, WDT_A_CLOCKSOURCE_SMCLK,
				WDT_A_CLOCKDIVIDER_32K);
		// Enable WDY interrupt.
		SFR_enableInterrupt(SFR_WATCHDOG_INTERVAL_TIMER_INTERRUPT);
		// Trigger WDT interrupt manually
		SFRIFG1 |= WDTIFG;

	}


	if(interruptTestRegister&TEST_RTC_INTERRUPT){
		RTC_enableInterrupt(RTC_BASE,RTC_OVERFLOW_INTERRUPT);
		RTCCTL |= (RTCIF);

	}


	if(interruptTestRegister&TEST_TIMER3_A1_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A3_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
		// Trigger timer manually
		TA3CCTL1 |= CCIFG;
	}


	if(interruptTestRegister&TEST_TIMER3_A0_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A3_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
		// Trigger timer manually
		TA3CCTL0 |= CCIFG;

	}


	if(interruptTestRegister&TEST_TIMER2_A1_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A2_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
		// Trigger timer manually
		TA2CCTL1 |= CCIFG;

	}


	if(interruptTestRegister&TEST_TIMER2_A0_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A2_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
		// Trigger timer manually
		TA2CCTL0 |= CCIFG;

	}


	if(interruptTestRegister&TEST_TIMER1_A1_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A1_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
		// Trigger timer manually
		TA1CCTL1 |= CCIFG;

	}


	if(interruptTestRegister&TEST_TIMER1_A0_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A1_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
		// Trigger timer manually
		TA1CCTL0 |= CCIFG;

	}


	if(interruptTestRegister&TEST_TIMER0_A1_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A0_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
		// Trigger timer manually
		TA0CCTL1 |= CCIFG;

	}


	if(interruptTestRegister&TEST_TIMER0_A0_INTERRUPT){
		// Configure timer interrupt
		Timer_A_enableCaptureCompareInterrupt(TIMER_A0_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
		// Trigger timer manually
		TA0CCTL0 |= CCIFG;

	}


	if(interruptTestRegister&TEST_UNMI_INTERRUPT){
		SFR_enableInterrupt(SFR_OSCILLATOR_FAULT_INTERRUPT);
		// Trigger UNMI interrupt manually
		SFRIFG1 |= (OFIFG);
	}


	if(interruptTestRegister&TEST_SYSNMI_INTERRUPT){
		SFR_enableInterrupt(SFR_VACANT_MEMORY_ACCESS_INTERRUPT);
		// Trigger SNMI interrupt manually
		SFRIFG1 |= (VMAIFG);

	}

	// Restore GIE status
	if(gieStatus)
	{
		__bis_SR_register(GIE);
	}
	else{
		__bic_SR_register(GIE);
	}

	if(gInterruptCounter!= interruptTestRegister){
		return TEST_FAILURE;
	}

	return SIG_INTERRUPT_TEST;
}


#pragma vector = CAPTIVATE_VECTOR
__interrupt void CAPTIVATE_ISR(void)
{
	return;
}

#pragma vector = PORT2_VECTOR
__interrupt void PORT2_ISR(void)
{
	gInterruptCounter |= TEST_PORT2_INTERRUPT;
	GPIO_clearInterrupt(GPIO_PORT_P2, GPIO_PIN7);
	GPIO_disableInterrupt(GPIO_PORT_P2, GPIO_PIN7);
}


#pragma vector = PORT1_VECTOR
__interrupt void PORT1_ISR(void)
{
	gInterruptCounter |= TEST_PORT1_INTERRUPT;
	GPIO_clearInterrupt(GPIO_PORT_P1, GPIO_PIN7);
	GPIO_disableInterrupt(GPIO_PORT_P1, GPIO_PIN7);
}


#pragma vector = ADC_VECTOR
__interrupt void ADC_ISR(void)
{
	gInterruptCounter |= TEST_ADC_INTERRUPT;
	ADC_clearInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT_FLAG);
	ADC_disableInterrupt(ADC_BASE,ADC_COMPLETED_INTERRUPT_FLAG);
}


#pragma vector = USCI_B0_VECTOR
__interrupt void USCI_B0_ISR(void)
{
	gInterruptCounter |= TEST_USCI_B0_INTERRUPT;
	EUSCI_B_SPI_disableInterrupt(EUSCI_B0_BASE, EUSCI_B_SPI_TRANSMIT_INTERRUPT);
}


#pragma vector = USCI_A1_VECTOR
__interrupt void USCI_A1_ISR(void)
{
	gInterruptCounter |= TEST_USCI_A1_INTERRUPT;
	EUSCI_A_SPI_disableInterrupt(EUSCI_A1_BASE, EUSCI_A_SPI_TRANSMIT_INTERRUPT);
}


#pragma vector = USCI_A0_VECTOR
__interrupt void USCI_A0_ISR(void)
{
	gInterruptCounter |= TEST_USCI_A0_INTERRUPT;
	EUSCI_A_SPI_disableInterrupt(EUSCI_A0_BASE, EUSCI_A_SPI_TRANSMIT_INTERRUPT);
}


#pragma vector = WDT_VECTOR
__interrupt void WDT_ISR(void)
{
	gInterruptCounter |= TEST_WDT_INTERRUPT;
	SFR_disableInterrupt(SFR_WATCHDOG_INTERVAL_TIMER_INTERRUPT);
	configureWDT();
}


#pragma vector = RTC_VECTOR
__interrupt void RTC_ISR(void)
{
	gInterruptCounter |= TEST_RTC_INTERRUPT;
	RTC_clearInterrupt(RTC_BASE,RTC_OVERFLOW_INTERRUPT_FLAG);
	RTC_disableInterrupt(RTC_BASE,RTC_OVERFLOW_INTERRUPT);
}


#pragma vector = TIMER3_A1_VECTOR
__interrupt void TIMER3_A1_ISR(void)
{
	gInterruptCounter |= TEST_TIMER3_A1_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A3_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_1);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A3_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
}



#pragma vector = TIMER3_A0_VECTOR
__interrupt void TIMER3_A0_ISR(void)
{
	gInterruptCounter |= TEST_TIMER3_A0_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A3_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_0);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A3_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
}


#pragma vector = TIMER2_A1_VECTOR
__interrupt void TIMER2_A1_ISR(void)
{
	gInterruptCounter |= TEST_TIMER2_A1_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A2_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_1);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A2_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
}


#pragma vector = TIMER2_A0_VECTOR
__interrupt void TIMER2_A0_ISR(void)
{
	gInterruptCounter |= TEST_TIMER2_A0_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A2_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_0);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A2_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
}


#pragma vector = TIMER1_A1_VECTOR
__interrupt void TIMER1_A1_ISR(void)
{
	gInterruptCounter |= TEST_TIMER1_A1_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A1_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_1);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A1_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
}


#pragma vector = TIMER1_A0_VECTOR
__interrupt void TIMER1_A0_ISR(void)
{
	gInterruptCounter |= TEST_TIMER1_A0_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A1_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_0);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A1_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
}


#pragma vector = TIMER0_A1_VECTOR
__interrupt void TIMER0_A1_ISR(void)
{
	gInterruptCounter |= TEST_TIMER0_A1_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A0_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_1);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A0_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_1);
}


#pragma vector = TIMER0_A0_VECTOR
__interrupt void TIMER0_A0_ISR(void)
{
	gInterruptCounter |= TEST_TIMER0_A0_INTERRUPT;
	Timer_A_clearCaptureCompareInterrupt(TIMER_A0_BASE,
			TIMER_A_CAPTURECOMPARE_REGISTER_0);
	Timer_A_disableCaptureCompareInterrupt(TIMER_A0_BASE,
				TIMER_A_CAPTURECOMPARE_REGISTER_0);
}

#pragma vector = UNMI_VECTOR
__interrupt void UNMI_ISR(void)
{
	gInterruptCounter |= TEST_UNMI_INTERRUPT;
	// Trigger UNMI interrupt manually
	SFRIFG1 &= ~(OFIFG);
	// Make sure oscillator flag is cleared by HW before continuing
	while(SFRIFG1 & OFIFG);
	SFR_disableInterrupt(SFR_OSCILLATOR_FAULT_INTERRUPT);

}

#pragma vector = SYSNMI_VECTOR
__interrupt void SYSNMI_ISR(void)
{
	gInterruptCounter |= TEST_SYSNMI_INTERRUPT;
	SFRIFG1 &= ~(VMAIFG);
	// Make sure vacant memory access flag is cleared by HW before continuing
	while(SFRIFG1 & OFIFG);
	SFR_disableInterrupt(SFR_VACANT_MEMORY_ACCESS_INTERRUPT);

}
